@@ -50,6 +50,7 @@ namespace FSpot {
 								     Dialog, Gtk.DialogFlags.Modal);
 			
 			Gtk.Entry password_entry = new Gtk.Entry ();
+			password_entry.ActivatesDefault = true;
 			password_entry.Visibility = false;
 
 			password_dialog.VBox.BorderWidth = 12;
@@ -57,6 +58,8 @@ namespace FSpot {
 			password_dialog.VBox.PackStart (new Gtk.Label (Mono.Posix.Catalog.GetString ("Enter Password for ") + email));
 			password_dialog.VBox.PackStart (password_entry);
 			password_dialog.AddButton (Gtk.Stock.Ok, Gtk.ResponseType.Ok);
+			password_dialog.HasSeparator = false;
+			password_dialog.DefaultResponse = Gtk.ResponseType.Ok;
 			password_dialog.ShowAll ();
 			password_dialog.Run ();
 			string password =  password_entry.Text;
@@ -107,10 +110,13 @@ namespace FSpot {
 
 			if (open && ids.Count != 0) {
 				string view_url = "http://www.flickr.com/tools/uploader_edit.gne?ids";
-				foreach (string id in ids)
-					view_url = view_url + "," + id;
+				bool first = true;
 
-				Gnome.Url.Show (view_url);
+				foreach (string id in ids) {
+					view_url = view_url + (first ? "=" : ",") + id;
+					first = false;
+				}
+				GnomeUtil.UrlShow (null, view_url);
 			}
 		}
 		
